/*****************************************************************************
 *            Created: 9/06/2012
 *          Copyright: (C) Copyright David Goodwin, 2012
 *            License: GNU General Public License
 *****************************************************************************
 *
 *   This is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This software is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this software; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 ****************************************************************************/

#include "mainwindow.h"
#include "ui_mainwindow.h"

#include <QByteArray>
#include <QMessageBox>
#include <QFileDialog>
#include <QtDebug>
#include <QFile>

#include "qextserialenumerator.h"

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);

    connect(ui->pbSend, SIGNAL(clicked()), this, SLOT(write()));
    connect(ui->pbSendFile, SIGNAL(clicked()), this, SLOT(writeFile()));
    connect(ui->pbOpen, SIGNAL(clicked()), this, SLOT(openPort()));

    connect(&port, SIGNAL(readyRead()), this, SLOT(portReadyRead()));

    statusMessage = new QLabel();
    ui->statusBar->addWidget(statusMessage);
    statusMessage->setText("Not connected");

    QList<QextPortInfo> ports = QextSerialEnumerator::getPorts();
    foreach(QextPortInfo port, ports) {
        ui->cmbPort->addItem(port.friendName,port.portName);
    }
// ---------------------baud_rates
    QList<quint32> baud_rates;
#if defined(Q_OS_UNIX)
    baud_rates << 50 << 75 << 134 << 150 << 200 << 1800; // POSIX ONLY
#  if defined(B76800)
    baud_rates << 76800; // POSIX ONLY
#  endif
#  if (defined(B230400) && defined(B4000000))
    baud_rates << 230400 << 460800 << 500000 << 576000 << 921600
               << 1000000 << 1152000 << 1500000 << 2000000 << 2500000
               << 3000000 << 3500000 << 4000000; // POSIX ONLY
#  endif
#endif //Q_OS_UNIX
#if defined(Q_OS_WIN)
    baud_rates << 14400 << 56000 << 128000 << 256000; // WINDOWS ONLY
#endif  //Q_OS_WIN
    baud_rates << 110 << 300 << 600 << 1200 << 2400 << 4800 << 9600 << 19200
               << 38400 << 57600 << 115200;

    qSort(baud_rates.begin(), baud_rates.end());

    foreach (quint32 baudRate, baud_rates) {
        ui->cmbBaudRate->addItem(QString::number(baudRate),baudRate);
    }

    int item = ui->cmbBaudRate->findText("9600");
    ui->cmbBaudRate->setCurrentIndex(item);

    // ---------------------Data bits
    ui->cmbDataBits->addItem("5",5);
    ui->cmbDataBits->addItem("6",6);
    ui->cmbDataBits->addItem("7",7);
    ui->cmbDataBits->addItem("8",8);

    item = ui->cmbDataBits->findText("8");
    ui->cmbDataBits->setCurrentIndex(item);

    // ---------------------- Parity
    ui->cmbParity->addItem("None");
    ui->cmbParity->addItem("Odd");
    ui->cmbParity->addItem("Even");
#if defined(Q_OS_WIN)
    ui->cmbParity->addItem("Mark");
#endif
    ui->cmbParity->addItem("Space");

    item = ui->cmbParity->findText("None");
    ui->cmbParity->setCurrentIndex(item);

    //-----------------------
    ui->cmbStopBits->addItem("1");
#if defined(Q_OS_WIN)
    ui->cmbStopBits->addItem("1.5");
#endif
    ui->cmbStopBits->addItem("2");

    item = ui->cmbStopBits->findText("1");
    ui->cmbStopBits->setCurrentIndex(item);

    //-----------------------
    ui->cmbFlowControl->addItem("Off");
    ui->cmbFlowControl->addItem("Hardware");
    ui->cmbFlowControl->addItem("XON/XOFF");
    item = ui->cmbFlowControl->findText("Off");
    ui->cmbFlowControl->setCurrentIndex(item);
}

MainWindow::~MainWindow()
{
    delete ui;
    if (port.isOpen())
        port.close();
}

void MainWindow::changeEvent(QEvent *e)
{
    QMainWindow::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

void MainWindow::openPort() {
    if (port.isOpen())
        port.close();

    QString portName = ui->cmbPort->itemData(
                ui->cmbPort->currentIndex()).toString();

    port.setPortName(portName);
    port.setBaudRate((BaudRateType)ui->cmbBaudRate->itemData(
                         ui->cmbBaudRate->currentIndex()).toUInt());
    port.setDataBits((DataBitsType)ui->cmbDataBits->itemData(
                         ui->cmbDataBits->currentIndex()).toInt());

    QString parityValue = ui->cmbParity->currentText();

    if (parityValue == "None")
        port.setParity(PAR_NONE);
    else if (parityValue == "Odd")
        port.setParity(PAR_ODD);
    else if (parityValue == "Even")
        port.setParity(PAR_EVEN);
    else if (parityValue == "Mark")
        port.setParity(PAR_MARK);
    else if (parityValue == "Space")
        port.setParity(PAR_SPACE);

    QString stopBitsVal = ui->cmbStopBits->currentText();
    if (stopBitsVal == "1")
        port.setStopBits(STOP_1);
    else if (stopBitsVal == "1.5")
        port.setStopBits(STOP_1_5);
    else if (stopBitsVal ==  "2")
        port.setStopBits(STOP_2);

    QString flowControlVal = ui->cmbFlowControl->currentText();
    if (flowControlVal == "Off")
        port.setFlowControl(FLOW_OFF);
    else if (flowControlVal == "Hardware")
        port.setFlowControl(FLOW_HARDWARE);
    else if (flowControlVal == "XON/XOFF")
        port.setFlowControl(FLOW_XONXOFF);

    if (port.open(QextSerialPort::ReadWrite)) {
        ui->pbSend->setEnabled(true);
        ui->pbSendFile->setEnabled(true);
        qDebug() << "Port open: " << portName;
        statusMessage->setText(ui->cmbPort->currentText());

    } else {
        QMessageBox::critical(this, "Error", "Failed to open serial port " + portName);
    }
}

void MainWindow::writeFile() {

    if (!port.isOpen()) return;

    QString filename = QFileDialog::getOpenFileName(this, "Send file");

    writeFile(filename);
}

void MainWindow::writeFile(QString filename) {

    if (filename.isEmpty()) return;
    if (!port.isOpen()) return;

    QFile file(filename);
    if (file.open(QFile::ReadOnly)) {
        QByteArray data = file.readAll();

        port.write(data);
        port.flush();

        file.close();
    } else {
        QMessageBox::critical(this, "Error", "Failed to open file for reading.");
    }
}

void MainWindow::write() {
    writeToPort(ui->pteData->toPlainText());
}


void MainWindow::writeToPort(QString data) {

    if (!port.isOpen()) {
        QMessageBox::critical(this, "Error", "Cannot write to port: Port not open.");
        return;
    }

    port.write(data.toAscii());
    qDebug() << "Wrote data to port: " << data;
}

void MainWindow::portReadyRead() {

    if (!port.isOpen()) return;

    QString currentValue = ui->pteResponse->toPlainText();
    QString newData = port.readAll();
    ui->pteResponse->setPlainText(currentValue + newData);

    qDebug() << "Read data: " << newData;
}
